#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "prologin.h"
#include "mission.h"
#include "utils.h"

static int mission;

/* zones de la carte
 *
 * ----------------------
 * |         |          |
 * |   0     |    2     |
 * |         |          |
 * |         |          |
 * ----------4-----------
 * |         |          |
 * |         |          |
 * |   1     |    3     |
 * |         |          |
 * ----------------------
 */
static int zone;

/* Milieu des zones */
static int midy() {
  if(zone == 2 || zone == 3)
    return mid(mid(taille_corps_y())) + mid(taille_corps_y());
  else if(zone == 1 || zone == 0)
    return mid(mid(taille_corps_y()));
  else
    return mid(taille_corps_y());
}

static int midx() {
  if(zone == 0 || zone == 2)
    return mid(mid(taille_corps_x()));
  else if(zone == 1 || zone == 3)
    return mid(mid(taille_corps_x())) + mid(taille_corps_x());
  else
    return mid(taille_corps_x());;

}

static void show() {
  switch(mission) {
  case DM_BOUGE:
    log2("=> bouge ");
    break;
  case DM_BOUGE_BACT:
    log2("=> bouge bact ");
    break;
  case DM_VIRUS:
    log2("=> attaque ");
    break;
  case DM_CELLULE:
    log2("=> cellule ");
    break;
  case DM_FLOOD:
    log2("=> flood ");
    break;
  case DM_BASTON:
    log2("=> baston ");
    break;
  }
}

static void affect(int m) {
  if(mission == m)
    log2("Meme mission -->");
  else {
    log2("Affectation -->");
    mission = m;
  }
  show();
}

static int is_bad_cellule(int x, int y) {
      int rv = etat_cellule(x, y);
      return (rv == CELL_STATE_INFECTED ||
	      rv == CELL_STATE_BEING_INFECTED);
}

static int check_cellule() {
  return plusproche(NULL, NULL, is_bad_cellule);
}

/* Attaquer les cellules proches dangereuses */
static void cellule(void) {
  int x, y;
  plusproche(&x, &y, is_bad_cellule);
  phago(x, y);
}

static int is_virus(int x, int y) {
  return regarde(x, y) == VIRUS;
}

/* nombre de virus proches */
static int check_virus() {
  return plusproche(NULL, NULL, is_virus);
}

/* Attaquer les virus proches */
static void virus(void) {
  int x, y;
  plusproche(&x, &y, is_virus);
  phago(x, y);
}

/* Mettre des anticorps partout */
static void flood(void) {
  emet_anticorps();
}

static int is_leuco(int x, int y) {
  int tid = equipe(identifiant_leucocyte(x,y));
  return (tid >= 0 && tid != mon_equipe());
}

/* retourne le nombre de leucocytes enemis proches */
static int check_leuco() {
  return plusproche(NULL, NULL, is_leuco);
}

/* Attaquer les leucocytes "enemis" proches */
static void baston(void) {
  int x, y;
  plusproche(&x, &y, is_leuco);
  phago(x, y);
}

/* Bouge en cercle autour de sa zone */
static void bouge() {
  static double a = 0;
  static int dx = -1;
  static int dy = -1;
  static int radius = -1;

  if(dx == -1 || dy == -1) {
    dx = midx(), dy = midy();

    if(zone > 3)
      radius =  taille_corps_x() / 3;
    else
      radius =  taille_corps_x() / 5;
  }
  if(abs(dx - getx()) + (abs(dy - gety())) < 4) {
    a += M_PI / 2.0f;
    dx = radius * cos(a) + midx();
    dy = radius * sin(a) + midy();
  } else
    deplace(dx, dy);
}

/* Se deplace la ou il y a le plus de bacteries */
static void bouge_bact() {
  int d = check_bacteries_rect(getx(), gety(), getx()+3, gety());
  int g = check_bacteries_rect(getx()-3, gety(), getx(), gety());
  int h = check_bacteries_rect(getx(), gety()-3, getx(), gety());
  int b = check_bacteries_rect(getx(), gety(), getx(), gety()+3);
  int max = -1;
  if(d > max)  max = d;
  if(g > max)  max = g;
  if(h > max)  max = h;
  if(b > max)  max = b;

  if(d == max) deplace(getx()+1, gety());
  else if(g == max) deplace(getx()-1, gety());
  else if(h == max) deplace(getx(), gety()-1);
  else if(b == max) deplace(getx(), gety()+1);
}

void init_mission(void) {
  log2("Init");
  affect(DM_BOUGE);
  zone =  (mon_identifiant() % taille_equipe(mon_equipe())) - 1;
  log("Zone %d\n", zone);
  show();
}

void affect_mission(void) {
  static int flood = 0;
  int midt = (battement_courant() > MAX_TURN / 2);
  log2("Affectation de la mission");

  /* Quand on commence a emetre des anticorps
   * on continue quelques temps, c'est rentable */
  if(flood) {
    flood++;
    flood %= 3;
    affect(DM_FLOOD);
    return;
  }

  /* Virus mechant pas beau */
  if(check_virus())
    affect(DM_VIRUS);
  /* elles non plus */
  else if(check_cellule())
    affect(DM_CELLULE);
  /* on tue pas les gens tout de suite */
  else if(check_leuco() && midt)
    affect(DM_BASTON);
  /* Bah, hop les anticorps */
  else if(check_bacteries() > 2 &&
	  bacteries_presentes(getx(), gety())) {
    affect(DM_FLOOD);
    flood = 1;
  /* ou qu'elles sont ? */
  } else if(check_bacteries() > 2)
    affect(DM_BOUGE_BACT);
  /* devrais aider en general */
  else if(midt && check_bacteries() >= 1)
    affect(DM_BOUGE_BACT);
  /* let,s go */
  else
    affect(DM_BOUGE);
}

void handle_mission(void) {
  log2("Fait la mission");
  show();

  switch(mission) {
  case DM_BOUGE:
    bouge();
    break;
  case DM_VIRUS:
    virus();
    break;
  case DM_CELLULE:
    cellule();
    break;
  case DM_FLOOD:
    flood();
    break;
  case DM_BASTON:
    baston();
    break;
  case DM_BOUGE_BACT:
    bouge_bact();
    break;
  }
}

